@@ -3,6 +3,15 @@
 This module contains clients to APIs and resources in the Riot Games ecosystem.
 """
 from typing import Any, Literal, TypedDict, Sequence
+from types import MethodType
+import abc
+import asyncio
+import functools
+import inspect
+import itertools
+import sys
+
+import aiohttp
 
 from .middlewares import (
     http_error_middleware,
@@ -17,13 +26,128 @@ from .schemas import (
     RiotAPISchema,
     MarlonAPISchema
 )
-from .base import Client, Middleware
+from .invocation import HttpMethod, Invocation
+from .middlewares import Middleware
 
 
 type _str = Sequence[str]
 
 
-class CDragonClient(Client):
+class BaseClient(abc.ABC):
+    """Base client class.
+
+    Inherit from this class to implement a client.
+    """
+
+    base_url: str
+    """Base URL, can be extended by `invoke`."""
+    default_headers: dict[str, str]
+    """Default params (ignores `...`), can be overwritten by `invoke`."""
+    default_params: dict[str, Any]
+    """Default header params, can be overwritten by `invoke`."""
+    default_queries: dict[str, str]
+    """Default query params, can be overwritten by `invoke`."""
+    middlewares: list[Middleware]
+    """Pre and post processors during `invoke`."""
+    session: aiohttp.ClientSession | None = None
+    """Context manager client session."""
+
+    def __init__(
+        self,
+        *,
+        base_url: str,
+        default_params: dict[str, Any] = {},
+        default_headers: dict[str, str] = {},
+        default_queries: dict[str, str] = {},
+        middlewares: list[Middleware] = [],
+    ) -> None:
+        self.base_url = base_url
+        self.default_headers = default_headers
+        self.default_params = default_params
+        self.default_queries = default_queries
+        self.middlewares = middlewares
+        async def run_invocation(invocation: Invocation):
+            return await invocation()
+        self.middleware_begin = run_invocation
+        for middleware in middlewares[::-1]:
+            self.middleware_begin = middleware(self.middleware_begin)
+            if not inspect.iscoroutinefunction(self.middleware_begin):
+                raise TypeError(f"{self.middleware_begin} is not a coroutine function")
+
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__} id={id(self)}>"
+
+    async def __aenter__(self):
+        """Context manager, in-context invocations will reuse a single `aiohttp.ClientSession`
+        improving performance and memory footprint.
+
+        Raises:
+            RuntimeError: When entering an already entered client.
+        """
+        if self.session:
+            raise RuntimeError(f"{self!r} has been already entered")
+        self.session = aiohttp.ClientSession()
+        return self
+
+    async def __aexit__(self, *_) -> None:
+        transports = 0
+        transports_closed = asyncio.Event()
+
+        def connection_lost(exc, orig_lost):
+            nonlocal transports
+            try:
+                orig_lost(exc)
+            finally:
+                transports -= 1
+                if transports == 0:
+                    transports_closed.set()
+
+        def eof_received(orig_eof_received):
+            try:
+                orig_eof_received()
+            except AttributeError:
+                pass
+
+        for conn in self.session.connector._conns.values():
+            for handler, _ in conn:
+                proto: asyncio.Protocol = getattr(handler.transport, "_ssl_protocol", None)
+                if proto is None:
+                    continue
+                transports += 1
+                orig_lost = proto.connection_lost
+                orig_eof_received = proto.eof_received
+                proto.connection_lost = functools.partial(
+                    connection_lost, orig_lost=orig_lost
+                )
+                proto.eof_received = functools.partial(
+                    eof_received, orig_eof_received=orig_eof_received
+                )
+        if transports == 0:
+            transports_closed.set()
+
+        await self.session.close()
+        await transports_closed.wait()
+        self.session = None
+
+    async def invoke(self, method: HttpMethod, path_or_url: str):
+        """Build an Invocation and send through the middlewares.
+
+        Params are automatically grabbed from the outer frame (ignores `...`).
+        The invoker client method is automatically grabbed from the outer frame
+        and passed to the instantiation of Invocation.
+        """
+        params = {}
+        for key, value in itertools.chain(self.default_params.items(), sys._getframe(1).f_locals.items()):
+            if key != "self" and value != ...:
+                params[key] = value
+        params["queries"] = {**self.default_queries, **params.get("queries", {})}
+        params["headers"] = {**self.default_headers, **params.get("headers", {})}
+        invoker: MethodType | None = getattr(self, sys._getframe(1).f_code.co_name, None)
+        invocation = Invocation(method, self.base_url + path_or_url, params, self.session, invoker=invoker)
+        return await self.middleware_begin(invocation)
+
+
+class CDragonClient(BaseClient):
     """Community Dragon Client.
 
     | Resources            | Support                    |
@@ -95,7 +219,7 @@ class CDragonClient(Client):
         return await self.invoke("GET", "/{patch}/cdragon/tft/{locale}.json")
 
 
-class DDragonClient(Client):
+class DDragonClient(BaseClient):
     """Data Dragon Client.
 
     | Resources            | Support                    |
@@ -146,7 +270,7 @@ class DDragonClient(Client):
         return await self.invoke("GET", "/{patch}/set{set}/{locale}/data/set{set}-{locale}.json")
 
 
-class MerakiCDNClient(Client):
+class MerakiCDNClient(BaseClient):
     """Meraki CDN Client.
 
     | Resources            | Support                    |
@@ -197,9 +321,12 @@ class MerakiCDNClient(Client):
 
     async def get_lol_item(self, *, id: int = ...) -> MerakiCDNSchema.LolItem:
         return await self.invoke("GET", "/lol/resources/latest/en-US/items/{id}.json")
+    
+    async def get_lol_champion_rates(self) -> MerakiCDNSchema.LolChampionRates:
+        return await self.invoke("GET", "/lol/resources/latest/en-US/championrates.json")
 
 
-class RiotAPIClient(Client):
+class RiotAPIClient(BaseClient):
     """Riot API Client.
 
     | Resources            | Support                    |
@@ -214,14 +341,15 @@ class RiotAPIClient(Client):
     async with RiotAPIClient(
         default_headers={"X-Riot-Token": <API_KEY>}
     ) as client:
-        summoner = await client.get_lol_summoner_v4_by_name(region="na1", name="Not a Whale")
+        account = await client.get_account_v1_by_riot_id(region="americas", game_name="200", tag_line="16384")
+        summoner = await client.get_lol_summoner_v4_by_puuid(region="na1", puuid=account["puuid"])
         assert summoner["summonerLevel"] > 200
     ```
     """
 
     Region = Literal[
         "americas", "europe", "asia", "sea", "esports",
-        "br1", "eun1", "euw1", "jp1", "kr", "la1", "la2",
+        "br1", "eun1", "euw1", "jp1", "kr", "la1", "la2", "me1",
         "na1", "oc1", "tr1", "ru", "ph2", "sg2", "th2", "tw2", "vn2",
         "ap", "br", "eu", "kr", "latam", "na",
     ] | _str
@@ -275,6 +403,9 @@ class RiotAPIClient(Client):
     async def get_lol_champion_v4_top_masteries_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.LolChampionV4Mastery]:
         return await self.invoke("GET", "/lol/champion-mastery/v4/champion-masteries/by-summoner/{summoner_id}/top")
 
+    async def get_lol_champion_v4_mastery_score_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> int:
+        return await self.invoke("GET", "/lol/champion-mastery/v4/scores/by-summoner/{summoner_id}")
+
     async def get_lol_champion_v4_mastery_by_puuid(self, *, region: Region = ..., puuid: str = ..., champion_id: int = ...) -> RiotAPISchema.LolChampionV4Mastery:
         return await self.invoke("GET", "/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/by-champion/{champion_id}")
 
@@ -284,6 +415,9 @@ class RiotAPIClient(Client):
     async def get_lol_champion_v4_top_masteries_by_puuid(self, *, region: Region = ..., puuid: str = ...) -> list[RiotAPISchema.LolChampionV4Mastery]:
         return await self.invoke("GET", "/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/top")
 
+    async def get_lol_champion_v4_mastery_score_by_puuid(self, *, region: Region = ..., puuid: str = ...) -> int:
+        return await self.invoke("GET", "/lol/champion-mastery/v4/scores/by-puuid/{puuid}")
+
     async def get_lol_clash_v1_players_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.LolClashV1Player]:
         return await self.invoke("GET", "/lol/clash/v1/players/by-summoner/{summoner_id}")
 
@@ -302,7 +436,7 @@ class RiotAPIClient(Client):
     async def get_lol_clash_v1_tournaments(self, *, region: Region = ...) -> list[RiotAPISchema.LolClashV1Tournament]:
         return await self.invoke("GET", "/lol/clash/v1/tournaments")
 
-    async def get_lol_league_v4_entries_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.LolLeagueV4LeagueEntry]:
+    async def get_lol_league_v4_entries_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.LolLeagueV4LeagueFullEntry]:
         return await self.invoke("GET", "/lol/league/v4/entries/by-summoner/{summoner_id}")
 
     async def get_lol_league_v4_challenger_league_by_queue(self, *, region: Region = ..., queue: str = ...) -> RiotAPISchema.LolLeagueV4League:
@@ -314,7 +448,9 @@ class RiotAPIClient(Client):
     async def get_lol_league_v4_master_league_by_queue(self, *, region: Region = ..., queue: str = ...) -> RiotAPISchema.LolLeagueV4League:
         return await self.invoke("GET", "/lol/league/v4/masterleagues/by-queue/{queue}")
 
-    async def get_lol_league_v4_entries_by_division(self, *, region: Region = ..., queue: str = ..., tier: str = ..., division: str = ..., queries: dict = {"page": 1}) -> list[RiotAPISchema.LolLeagueV4LeagueEntry]:
+    async def get_lol_league_v4_entries_by_division(
+        self, *, region: Region = ..., queue: str = ..., tier: str = ..., division: str = ..., queries: dict = {"page": 1}
+    ) -> list[RiotAPISchema.LolLeagueV4LeagueFullEntry]:
         return await self.invoke("GET", "/lol/league/v4/entries/{queue}/{tier}/{division}")
 
     async def get_lol_league_v4_league(self, *, region: Region = ..., id: str = ...) -> RiotAPISchema.LolLeagueV4League:
@@ -335,6 +471,12 @@ class RiotAPIClient(Client):
     async def get_lol_spectator_v4_featured_games(self, *, region: Region = ...) -> RiotAPISchema.LolSpectatorV4GameList:
         return await self.invoke("GET", "/lol/spectator/v4/featured-games")
 
+    async def get_lol_spectator_v5_active_game_by_summoner(self, *, region: Region = ..., puuid: str = ...) -> RiotAPISchema.LolSpectatorV5Game:
+        return await self.invoke("GET", "/lol/spectator/v5/active-games/by-summoner/{puuid}")
+
+    async def get_lol_spectator_v5_featured_games(self, *, region: Region = ...) -> RiotAPISchema.LolSpectatorV5GameList:
+        return await self.invoke("GET", "/lol/spectator/v5/featured-games")
+
     async def get_lol_status_v4_platform_data(self, *, region: Region = ...) -> RiotAPISchema.StatusV1PlatformData:
         return await self.invoke("GET", "/lol/status/v4/platform-data")
 
@@ -355,7 +497,7 @@ class RiotAPIClient(Client):
 
     # Teamfight Tactics Endpoints
 
-    async def get_tft_league_v1_entries_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.TftLeagueV1LeagueEntry]:
+    async def get_tft_league_v1_entries_by_summoner(self, *, region: Region = ..., summoner_id: str = ...) -> list[RiotAPISchema.TftLeagueV1LeagueFullEntry]:
         return await self.invoke("GET", "/tft/league/v1/entries/by-summoner/{summoner_id}")
 
     async def get_tft_league_v1_challenger_league(self, *, region: Region = ..., queries: dict = {}) -> RiotAPISchema.TftLeagueV1League:
@@ -367,7 +509,9 @@ class RiotAPIClient(Client):
     async def get_tft_league_v1_master_league(self, *, region: Region = ..., queries: dict = {}) -> RiotAPISchema.TftLeagueV1League:
         return await self.invoke("GET", "/tft/league/v1/master")
 
-    async def get_tft_league_v1_entries_by_division(self, *, region: Region = ..., tier: str = ..., division: str = ..., queries: dict = {"page": 1}) -> list[RiotAPISchema.TftLeagueV1LeagueEntry]:
+    async def get_tft_league_v1_entries_by_division(
+        self, *, region: Region = ..., tier: str = ..., division: str = ..., queries: dict = {"page": 1}
+    ) -> list[RiotAPISchema.TftLeagueV1LeagueFullEntry]:
         return await self.invoke("GET", "/tft/league/v1/entries/{tier}/{division}")
 
     async def get_tft_league_v1_league(self, *, region: Region = ..., id: str = ...) -> RiotAPISchema.TftLeagueV1League:
@@ -429,7 +573,7 @@ class RiotAPIClient(Client):
         return await self.invoke("GET", "/val/status/v1/platform-data")
 
 
-class MarlonAPIClient(Client):
+class MarlonAPIClient(BaseClient):
     """Marlon API Client.
 
     | Resources            | Support                    |
