@@ -53,6 +53,7 @@ class CDragonSchema:
         "isLegacy": bool,
         "splashVideoPath": str | None,
         "collectionSplashVideoPath": str | None,
+        "collectionCardHoverVideoPath": str | None,
         "featuresText": str | None,
         "chromaPath": str | None,
         "emblems": str | None,
@@ -226,14 +227,15 @@ class CDragonSchema:
         "name": str,
         "unique": bool
     })
+    TftDataTftSetChampionAbilityVariable = TypedDict("TftDataTftSetChampionAbilityVariable", {
+        "name": str,
+        "value": list[float]
+    })
     TftDataTftSetChampionAbility = TypedDict("TftDataTftSetChampionAbility", {
         "desc": str,
         "icon": str,
         "name": str,
-        "variables": list[TypedDict("Variable", {
-            "name": str,
-            "value": list[float]
-        })]
+        "variables": list[TftDataTftSetChampionAbilityVariable]
     })
     TftDataTftSetChampionStats = TypedDict("TftDataTftSetChampionStats", {
         "armor": float,
@@ -491,6 +493,7 @@ class MerakiCDNSchema:
         "attackType": str,
         "adaptiveType": str,
         "stats": LolChampionStats,
+        "positions": list[str],
         "roles": list[str],
         "attributeRatings": LolChampionAttributeRatings,
         "abilities": LolChampionAbilities,
@@ -580,6 +583,18 @@ class MerakiCDNSchema:
         "shop": LolItemShop,
         "iconOverlay": bool
     })
+    LolPlayRate = TypedDict("LolPlayRate", {"playRate": float})
+    LolChampionRate = TypedDict("LolChampionRate", {
+        "TOP": LolPlayRate,
+        "JUNGLE": LolPlayRate,
+        "MIDDLE": LolPlayRate,
+        "BOTTOM": LolPlayRate,
+        "UTILITY": LolPlayRate,
+    })
+    LolChampionRates = TypedDict("LolChampionRates", {
+        "data": dict[str, LolChampionRate],
+        "patch": str
+    })
 
 
 class RiotAPISchema:
@@ -604,6 +619,17 @@ class RiotAPISchema:
         "freeChampionIdsForNewPlayers": list[int],
         "maxNewPlayerLevel": int
     })
+    LolChampionV4MasteryMilestoneRewardConfig = TypedDict("LolChampionV4MasteryMilestoneRewardConfig", {
+        "maximumReward": int,
+        "rewardType": str,
+        "rewardValue": str,
+    })
+    LolChampionV4MasteryMilestone = TypedDict("LolChampionV4MasteryMilestone", {
+        "bonus": bool,
+        "requireGradeCounts": dict[str, int],
+        "rewardConfig": LolChampionV4MasteryMilestoneRewardConfig,
+        "rewardMarks": int
+    })
     LolChampionV4Mastery = TypedDict("LolChampionV4Mastery", {
         "puuid": str,
         "championId": int,
@@ -612,9 +638,12 @@ class RiotAPISchema:
         "lastPlayTime": int,
         "championPointsSinceLastLevel": int,
         "championPointsUntilNextLevel": int,
+        "championSeasonMilestone": int,
+        "markRequiredForNextLevel": int,
+        "milestoneGrades": NotRequired[list[str]],
+        "nextSeasonMilestone": LolChampionV4MasteryMilestone,
         "chestGranted": bool,
         "tokensEarned": int,
-        "summonerId": str
     })
     LolClashV1Player = TypedDict("LolClashV1Player", {
         "summonerId": str,
@@ -643,7 +672,7 @@ class RiotAPISchema:
         "themeId": int,
         "nameKey": str,
         "nameKeySecondary": str,
-        "schedule": LolClashV1TournamentSchedule
+        "schedule": list[LolClashV1TournamentSchedule]
     })
     LolLeagueV4LeagueEntryMiniSeries = TypedDict("LolLeagueV4LeagueEntryMiniSeries", {
         "losses": int,
@@ -652,11 +681,23 @@ class RiotAPISchema:
         "wins": int,
     })
     LolLeagueV4LeagueEntry = TypedDict("LolLeagueV4LeagueEntry", {
-        "leagueId": NotRequired[str],
         "summonerId": str,
-        "summonerName": str,
-        "queueType": NotRequired[str],
-        "tier": NotRequired[str],
+        "summonerName": NotRequired[str],
+        "rank": str,
+        "leaguePoints": int,
+        "wins": int,
+        "losses": int,
+        "hotStreak": bool,
+        "veteran": bool,
+        "freshBlood": bool,
+        "inactive": bool,
+    })
+    LolLeagueV4LeagueFullEntry = TypedDict("LolLeagueV4LeagueFullEntry", {
+        "leagueId": str,
+        "summonerId": str,
+        "summonerName": NotRequired[str],
+        "queueType": str,
+        "tier": str,
         "rank": str,
         "leaguePoints": int,
         "wins": int,
@@ -811,7 +852,7 @@ class RiotAPISchema:
         "highestWardKills": NotRequired[int],
         "soloTurretsLategame": NotRequired[int],
         "thirdInhibitorDestroyedTime": NotRequired[float],
-    }, total=False) | dict[str, int | float]
+    }, total=False)
     LolMatchV5MatchInfoParticipantPerksStatPerks = TypedDict("LolMatchV5MatchInfoParticipantPerksStatPerks", {
         "defense": int,
         "flex": int,
@@ -821,8 +862,8 @@ class RiotAPISchema:
         "perk": int,
         "var1": int,
         "var2": int,
-        "var3": int}
-    )
+        "var3": int
+    })
     LolMatchV5MatchInfoParticipantPerksStyle = TypedDict("LolMatchV5MatchInfoParticipantPerksStyle", {
         "description": str,
         "selections": list[LolMatchV5MatchInfoParticipantPerksStyleSelection],
@@ -846,15 +887,29 @@ class RiotAPISchema:
         "playerScore8": float,
         "playerScore9": float,
     })
+    LolMatchV5MatchInfoParticipantMissionsAlt = TypedDict("LolMatchV5MatchInfoParticipant", {
+        "PlayerScore0": float,
+        "PlayerScore1": float,
+        "PlayerScore10": float,
+        "PlayerScore11": float,
+        "PlayerScore2": float,
+        "PlayerScore3": float,
+        "PlayerScore4": float,
+        "PlayerScore5": float,
+        "PlayerScore6": float,
+        "PlayerScore7": float,
+        "PlayerScore8": float,
+        "PlayerScore9": float,
+    })
     LolMatchV5MatchInfoParticipant = TypedDict("LolMatchV5MatchInfoParticipant", {
         "allInPings": int,
         "assistMePings": int,
         "assists": int,
-        "baitPings": int,
+        "baitPings": NotRequired[int],
         "baronKills": int,
         "basicPings": int,
         "bountyLevel": int,
-        "challenges": NotRequired[LolMatchV5MatchInfoParticipantChallenges],
+        "challenges": NotRequired[LolMatchV5MatchInfoParticipantChallenges | dict[str, int | float]],
         "champExperience": int,
         "champLevel": int,
         "championId": int,
@@ -906,7 +961,7 @@ class RiotAPISchema:
         "magicDamageDealt": int,
         "magicDamageDealtToChampions": int,
         "magicDamageTaken": int,
-        "missions": NotRequired[LolMatchV5MatchInfoParticipantMissions],
+        "missions": NotRequired[LolMatchV5MatchInfoParticipantMissions | LolMatchV5MatchInfoParticipantMissionsAlt],
         "needVisionPings": int,
         "neutralMinionsKilled": int,
         "nexusKills": int,
@@ -959,7 +1014,7 @@ class RiotAPISchema:
         "summoner2Id": int,
         "summonerId": str,
         "summonerLevel": int,
-        "summonerName": str,
+        "summonerName": NotRequired[str],
         "teamEarlySurrendered": bool,
         "teamId": int,
         "teamPosition": str,
@@ -1000,7 +1055,7 @@ class RiotAPISchema:
         "baron": LolMatchV5MatchTeamObjective,
         "champion": LolMatchV5MatchTeamObjective,
         "dragon": LolMatchV5MatchTeamObjective,
-        "horde": LolMatchV5MatchTeamObjective,
+        "horde": NotRequired[LolMatchV5MatchTeamObjective],
         "inhibitor": LolMatchV5MatchTeamObjective,
         "riftHerald": LolMatchV5MatchTeamObjective,
         "tower": LolMatchV5MatchTeamObjective
@@ -1026,7 +1081,8 @@ class RiotAPISchema:
         "platformId": str,
         "queueId": int,
         "teams": list[LolMatchV5MatchInfoTeam],
-        "tournamentCode": str
+        "tournamentCode": str,
+        "endOfGameResult": str
     })
     LolMatchV5Match = TypedDict("LolMatchV5Match", {
         "metadata": LolMatchV5MatchMetadata,
@@ -1131,21 +1187,9 @@ class RiotAPISchema:
         "skillSlot": NotRequired[int],
         "realTimestamp": NotRequired[int],
     })
-    LolMatchV5MatchTimelineInfoFrameParticipantFrames = TypedDict("LolMatchV5MatchTimelineInfoFrameParticipantFrames", {
-        "1": LolMatchV5MatchTimelineParticipantFrame,
-        "2": LolMatchV5MatchTimelineParticipantFrame,
-        "3": LolMatchV5MatchTimelineParticipantFrame,
-        "4": LolMatchV5MatchTimelineParticipantFrame,
-        "5": LolMatchV5MatchTimelineParticipantFrame,
-        "6": LolMatchV5MatchTimelineParticipantFrame,
-        "7": LolMatchV5MatchTimelineParticipantFrame,
-        "8": LolMatchV5MatchTimelineParticipantFrame,
-        "9": LolMatchV5MatchTimelineParticipantFrame,
-        "10": LolMatchV5MatchTimelineParticipantFrame
-    })
     LolMatchV5MatchTimelineInfoFrame = TypedDict("LolMatchV5MatchTimelineInfoFrame", {
         "events": list[LolMatchV5MatchTimelineInfoFrameEvent],
-        "participantFrames": LolMatchV5MatchTimelineInfoFrameParticipantFrames,
+        "participantFrames": dict[str, LolMatchV5MatchTimelineParticipantFrame],
         "timestamp": int
     })
     LolMatchV5MatchTimelineInfoParticipants = TypedDict("LolMatchV5MatchTimelineInfoParticipants", {
@@ -1156,7 +1200,8 @@ class RiotAPISchema:
         "frameInterval": int,
         "frames": list[LolMatchV5MatchTimelineInfoFrame],
         "gameId": int,
-        "participants": list[LolMatchV5MatchTimelineInfoParticipants]
+        "participants": list[LolMatchV5MatchTimelineInfoParticipants],
+        "endOfGameResult": str
     })
     LolMatchV5MatchTimeline = TypedDict("LolMatchV5MatchTimeline", {
         "metadata": LolMatchV5MatchTimelineMetadata,
@@ -1177,7 +1222,7 @@ class RiotAPISchema:
         "spell2Id": int,
         "championId": int,
         "profileIconId": int,
-        "summonerName": str,
+        "summonerName": NotRequired[str],
         "bot": bool
     })
     LolSpectatorV4GameObservers = TypedDict("LolSpectatorV4GameObservers", {
@@ -1200,11 +1245,45 @@ class RiotAPISchema:
         "gameList": list[LolSpectatorV4Game],
         "clientRefreshInterval": int
     })
+    LolSpectatorV5GameParticipant = TypedDict("LolSpectatorV5GameParticipant", {
+        "gameCustomizationObjects": NotRequired[list[str]],
+        "perks": NotRequired[LolSpectatorV4GameParticipantPerks],
+        "puuid": str,
+        "summonerId": str,
+        "teamId": int,
+        "spell1Id": int,
+        "spell2Id": int,
+        "championId": int,
+        "profileIconId": int,
+        "riotId": str,
+        "bot": bool,
+    })
+    LolSpectatorV5GameBannedChampion = TypedDict("LolSpectatorV4Game", {
+        "championId": int,
+        "pickTurn": int,
+        "teamId": int,
+    })
+    LolSpectatorV5Game = TypedDict("LolSpectatorV5Game", {
+        "gameId": int,
+        "mapId": int,
+        "gameMode": str,
+        "gameType": str,
+        "gameQueueConfigId": int,
+        "participants": list[LolSpectatorV5GameParticipant],
+        "observers": LolSpectatorV4GameObservers,
+        "platformId": str,
+        "bannedChampions": list[LolSpectatorV5GameBannedChampion],
+        "gameStartTime": NotRequired[int],
+        "gameLength": int
+    })
+    LolSpectatorV5GameList = TypedDict("LolSpectatorV5GameList", {
+        "gameList": list[LolSpectatorV5Game],
+    })
     LolSummonerV4Summoner = TypedDict("SummonerV4Summoner", {
         "id": str,
         "accountId": str,
         "puuid": str,
-        "name": str,
+        "name": NotRequired[str],
         "profileIconId": int,
         "revisionDate": int,
         "summonerLevel": int
@@ -1213,12 +1292,24 @@ class RiotAPISchema:
     # Teamfight Tactics Types
 
     TftLeagueV1LeagueEntry = TypedDict("TftLeagueV1LeagueEntry", {
-        "leagueId": NotRequired[str],
+        "summonerId": str,
+        "summonerName": NotRequired[str],
+        "rank": str,
+        "leaguePoints": int,
+        "wins": int,
+        "losses": int,
+        "hotStreak": bool,
+        "veteran": bool,
+        "freshBlood": bool,
+        "inactive": bool,
+    })
+    TftLeagueV1LeagueFullEntry = TypedDict("TftLeagueV1LeagueFullEntry", {
+        "leagueId": str,
         "puuid": str,
         "summonerId": str,
-        "summonerName": str,
-        "queueType": NotRequired[str],
-        "tier": NotRequired[str],
+        "summonerName": NotRequired[str],
+        "queueType": str,
+        "tier": str,
         "rank": str,
         "leaguePoints": int,
         "wins": int,
@@ -1234,7 +1325,7 @@ class RiotAPISchema:
         "leagueId": NotRequired[str],
         "queue": NotRequired[str],
         "name": NotRequired[str],
-        "entries": list[LolLeagueV4LeagueEntry]
+        "entries": list[TftLeagueV1LeagueEntry]
     })
     TftMatchV1MatchMetadata = TypedDict("TftMatchV1MatchMetadata", {
         "data_version": str,
@@ -1261,6 +1352,56 @@ class RiotAPISchema:
         "rarity": int,
         "tier": int
     })
+    TftMatchV1MatchInfoParticipantMission = TypedDict("TftMatchV1MatchInfoParticipantMission", {
+        "Assists": int,
+        "DamageDealt": int,
+        "DamageDealtToObjectives": int,
+        "DamageDealtToTurrets": int,
+        "DamageTaken": int,
+        "Deaths": int,
+        "DoubleKills": int,
+        "GoldEarned": int,
+        "GoldSpent": int,
+        "InhibitorsDestroyed": int,
+        "KillingSprees": int,
+        "Kills": int,
+        "LargestKillingSpree": int,
+        "LargestMultiKill": int,
+        "MagicDamageDealt": int,
+        "MagicDamageDealtToChampions": int,
+        "MagicDamageTaken": int,
+        "NeutralMinionsKilledTeamJungle": int,
+        "PentaKills": int,
+        "PhysicalDamageDealt": int,
+        "PhysicalDamageDealtToChampions": int,
+        "PhysicalDamageTaken": int,
+        "PlayerScore0": int,
+        "PlayerScore1": int,
+        "PlayerScore10": int,
+        "PlayerScore11": int,
+        "PlayerScore2": int,
+        "PlayerScore3": int,
+        "PlayerScore4": int,
+        "PlayerScore5": int,
+        "PlayerScore6": int,
+        "PlayerScore9": int,
+        "QuadraKills": int,
+        "Spell1Casts": int,
+        "Spell2Casts": int,
+        "Spell3Casts": int,
+        "Spell4Casts": int,
+        "SummonerSpell1Casts": int,
+        "TimeCCOthers": int,
+        "TotalDamageDealtToChampions": int,
+        "TotalMinionsKilled": int,
+        "TripleKills": int,
+        "TrueDamageDealt": int,
+        "TrueDamageDealtToChampions": int,
+        "TrueDamageTaken": int,
+        "UnrealKills": int,
+        "VisionScore": int,
+        "WardsKilled": int,
+    })
     TftMatchV1MatchInfoParticipant = TypedDict("TftMatchV1MatchInfoParticipant", {
         "augments": list[str],
         "companion": TftMatchV1MatchInfoParticipantCompanion,
@@ -1273,7 +1414,8 @@ class RiotAPISchema:
         "time_eliminated": float,
         "total_damage_to_players": int,
         "traits": list[TftMatchV1MatchInfoParticipantTrait],
-        "units": list[TftMatchV1MatchInfoParticipantUnit]
+        "units": list[TftMatchV1MatchInfoParticipantUnit],
+        "missions": TftMatchV1MatchInfoParticipantMission,
     })
     TftMatchV1MatchInfo = TypedDict("TftMatchV1MatchInfo", {
         "game_datetime": int,
@@ -1283,7 +1425,12 @@ class RiotAPISchema:
         "queue_id": int,
         "tft_game_type": str,
         "tft_set_core_name": str,
-        "tft_set_number": int
+        "tft_set_number": int,
+        "endOfGameResult": str,
+        "gameCreation": int,
+        "gameId": int,
+        "mapId": int,
+        "queueId": int,
     })
     TftMatchV1Match = TypedDict("TftMatchV1Match", {
         "metadata": TftMatchV1MatchMetadata,
@@ -1315,6 +1462,7 @@ class RiotAPISchema:
         "order_of_play": int
     })
     LorMatchV1MatchInfo = TypedDict("LorMatchV1MatchInfo", {
+        "game_format": str,
         "game_mode": str,
         "game_type": str,
         "game_start_time_utc": str,
@@ -1477,6 +1625,11 @@ class RiotAPISchema:
         "roundsWon": int,
         "numPoints": int
     })
+    ValMatchV1MatchRoundResultPlayerStatFinishingDamage = TypedDict("ValMatchV1MatchRoundResultPlayerStatFinishingDamage", {
+        "damageType": str,
+        "damageItem": str,
+        "isSecondaryFireMode": bool
+    })
     ValMatchV1MatchRoundResultPlayerStatKill = TypedDict("ValMatchV1MatchRoundResultPlayerStatKill", {
         "timeSinceGameStartMillis": int,
         "timeSinceRoundStartMillis": int,
@@ -1485,11 +1638,7 @@ class RiotAPISchema:
         "victimLocation": ValMatchV1MatchLocation,
         "assistants": list[str],
         "playerLocations": list[ValMatchV1MatchPlayerLocation],
-        "finishingDamage": TypedDict("FinishingDamage", {
-            "damageType": str,
-            "damageItem": str,
-            "isSecondaryFireMode": bool
-        })
+        "finishingDamage": ValMatchV1MatchRoundResultPlayerStatFinishingDamage
     })
     ValMatchV1MatchRoundResultPlayerStatDamage = TypedDict("ValMatchV1MatchRoundResultPlayerStatDamage", {
         "receiver": str,
